@@ -8,6 +8,14 @@ class HDF5writer:
         self.hdffile_path = Path(path_parent, filename + ".hdf5")
 
     def write_file(self, rock_collection: list, element_list: list, init_parameters_TherCaller, author_name: str = "philip hartmeier"):
+        """_summary_
+
+        Args:
+            rock_collection (list): _description_
+            element_list (list): _description_
+            init_parameters_TherCaller (pytheriak.wrapper.TherCaller): _description_
+            author_name (str, optional): _description_. Defaults to "philip hartmeier".
+        """
         with h5py.File(self.hdffile_path, "w") as hdf_file:
             hdf_file.attrs["date"] = str(date.today())
             hdf_file.attrs["author"] = author_name
@@ -30,7 +38,7 @@ class HDF5writer:
                 rock_group.attrs["G_system"] = float(rock.g_system)
                 rock_group.attrs["G_system_per_mol_of_input"] = float(rock.g_system_per_mol_of_input)
                 # save compositional data (bulk) as dataset, element_list acts as globally valid look-up table for idx --> element
-                rock_group.create_dataset("bulk_composition_in_mol", data=rock.bulk_composition_n)
+                rock_group.create_dataset("bulk_composition_in_mol", data=rock.bulk_composition_moles)
                 rock_group.create_dataset("bulk_composition_in_mol_percent", data=rock.bulk_composition_mol_percent)
 
                 # make a sub-group for mineral_assemblage of the Rock()
@@ -45,7 +53,7 @@ class HDF5writer:
                     mineral_group.attrs["volume_percent_of_total_solids"] = mineral.vol_percent
                     mineral_group.attrs["density"] = mineral.density
                     # save mineral composition as dataset, element_list acts as globally valid look-up table for idx --> element
-                    mineral_group.create_dataset("phase_composition_apfu", data=mineral.composition)
+                    mineral_group.create_dataset("phase_composition_apfu", data=mineral.composition_apfu)
                     mineral_group.create_dataset("phase_composition_moles", data=mineral.composition_moles)
 
                 # make a sub-group for fluid_assemblage of the Rock()
@@ -59,7 +67,7 @@ class HDF5writer:
                     fluid_group.attrs["volume"] = fluid.vol
                     fluid_group.attrs["density"] = fluid.density
 
-                    fluid_group.create_dataset("phase_composition_apfu", data=fluid.composition)
+                    fluid_group.create_dataset("phase_composition_apfu", data=fluid.composition_apfu)
                     fluid_group.create_dataset("phase_composition_moles", data=fluid.composition_moles)
 
                 # save deltaG of all metastable minerals for the Rock()
